# frozen_string_literal: true
require 'json'
module Hermes
  module Api
    module Dmo
      # checker Api::Dmo::HermesController avant d'ajouter des methodes
      class LessorsController < Hermes::Api::Dmo::ApplicationController
        set_instance_for :objectives

        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $eq
          $regex
          $all
        ]

        ACCEPTABLE_PARAMS = %w[
          ids
          als_id
          created_at
          updated_at
          title
          siret
          siren
          is_root
          root_id
          parent_id
          id_rex
          id_cristal
          api_id
          rpls_id
          affiliate
          housing_offer_ids
          lessor_objective_ids
          group_id
          supervisor_group_id
          manager_group_id
        ]

        REGEX_PARAMS = %w[
          title
          siret
          siren
        ]

        OBJECTIVES_ACCEPTABLE_PARAMS = %w[
          created_at
          updated_at
          activation_date
          department
          district
          threshold_type
          priority_public
          new_offer
          target_by_typologies
          realized_by_typologies
          present_typologies
          agency_id
        ]

        OBJECTIVES_REGEX_PARAMS = %w[
          district
        ]

        MUTATE_PARAMS = %i[
          title
          siret
          siren
          id_rex
          id_cristal
          api_id
          rpls_id
          affiliate
          housing_offer_ids
          lessor_objective_ids
        ]
        mutate_params_only *MUTATE_PARAMS

        def index
          acceptable_params *ACCEPTABLE_PARAMS
          regex_params *REGEX_PARAMS
          valid_operators *VALID_OPERATORS
          super
        end

        def objective_handle_params
          acceptable_params *OBJECTIVES_ACCEPTABLE_PARAMS
          regex_params *OBJECTIVES_REGEX_PARAMS
          valid_operators *VALID_OPERATORS
        end
        
        def is_super_admin
          current_user_roles.include?('superadministrateur')
        end

        def admin_search
          if is_super_admin
            if !params['lessor_params'].nil?
              params_lessor = JSON.parse(params['lessor_params'])
              lessor_query = {}

              unless (id = params_lessor['id']).blank?
                lessor_query[:id] = id
              end

              unless (id_cristal = params_lessor['id_cristal']).blank?
                lessor_query[:id_cristal] = id_cristal
              end
        
              unless (siren = params_lessor['siren']).blank?
                lessor_query[:siren] = siren
              end
        
              unless (siret = params_lessor['siret']).blank?
                lessor_query[:siret] = siret
              end

              unless (title = params_lessor['title']).blank?
                lessor_query[:title] = title
              end
        
              lessor_list = Hermes::Lessor.where(lessor_query).to_a


              if !params['user_params'].blank?
                user_params = JSON.parse(params['user_params'])
                
                new_list = []
                unless lessor_list.blank?
                  lessor_list.each do |lessor|
                    users = []
                    lessor_response = {
                      id: lessor.id,
                      id_cristal: lessor.id_cristal,
                      siren: lessor.siren,
                      siret: lessor.siret,
                      title: lessor.title
                    }

                    users_found_by_id = []

                    lessor.all_users.each do |uid|
                        user = User.find(uid)
                        users_found_by_id << user
                    end   
                    
                    user_params = JSON.parse(params['user_params'])

                    user_query = {}
                    unless (id = user_params['id']).blank?
                        user_query[:id] = id
                    end
              
                    unless (first_name = user_params['first_name']).blank?
                        user_query[:first_name] = first_name
                    end
              
                    unless (last_name = user_params['last_name']).blank?
                      user_query[:last_name] = last_name
                    end

                    unless (auth_roles = user_params['auth_roles']).blank?
                      user_query[:auth_roles] = auth_roles
                    end
              
        
              
                    user_list = User.where(user_query).to_a
      
      
                    
                    users_final = []
                    unless user_list.blank?
                      users_found_by_id.each do |user_found_by_id|
                          user_list.each do |user_found_by_filters|
                              if user_found_by_filters == user_found_by_id
                                  users_final << user_found_by_id
                              end 
                          end 
                      end
                    end 

                    users_final.each do |user|
                        lessor_user = {
                          id: user.id,
                          first_name: user.first_name,
                          last_name: user.last_name,
                          auth_roles: user.auth_roles
                        }
                        users << lessor_user
                    end 

                    new_entrie = {"lessor" => lessor_response, "user" => users}
                    new_list << new_entrie
                  end
                end 

                render json: {status: "success", code: 200, message: {"lessors_and_users" => new_list}}
              else

                new_list = []
                unless lessor_list.nil?
                  lessor_list.each do |lessor|
                    users = []
                    lessor_response = {
                      id: lessor.id,
                      id_cristal: lessor.id_cristal,
                      siren: lessor.siren,
                      siret: lessor.siret,
                      title: lessor.title
                    }

                    users_found_by_id = []
                    lessor.all_users.each do |uid|
                      user = User.find(uid)
                      users_found_by_id << user
                    end   

                    users_found_by_id.each do |user|
                      lessor_user = {
                        id: user.id,
                        first_name: user.first_name,
                        last_name: user.last_name,
                        auth_roles: user.auth_roles
                      }
                      users << lessor_user
                    end 
                    new_entrie = {"lessor" => lessor_response, "user" => users}
                    new_list << new_entrie
                  end 
                end 
                render json: {status: "success", code: 200, message:  {"lessors_and_users" => new_list}}
              end
            else 
              render json: {status: "unauthorized", code: 401, message: "no lessor params detected"}
            end 
          else 
            render json: {status: "unauthorized", code: 401, message: "user is no superadmin"}
          end  
        end

        def paginate_objectives(objectives)
          objective_handle_params
          objectives = objectives.where(build_query(Hermes::LessorObjective))
                      .order(order_params)
          Rails.logger.debug "TEST #{objectives.selector}"
          paginate(
            objectives,
            each_serializer: Hermes::LessorObjectiveSerializer
          )
        end

        def all_objectives
          objectives = @instance.lessor_objectives
          paginate_objectives(objectives)
        end

        def active_objectives
          objectives = @instance.lessor_objectives.actives
          paginate_objectives(objectives)
        end

        def switch_manual_mode
          if is_super_admin
            lessor = Hermes::Lessor.find(params[:id])
            if lessor.switch_manual_mode
              render json: {status: "success", code: 200, message: "manual mode switch to #{lessor.manual_mode}"}
            else
              render json: {status: "not found", code: 404, message: 'lessor not found'}
            end 
          else 
            render json: {status: "unauthorized", code: 401, message: "user is not a superadmin"}
          end 
        end 

        def current_operation_names
          operation_names = @instance.current_operation_names
          render :json => operation_names
        end

        def add_user
          if @instance.writable_by? current_user
            user = ::User.find params[:data][:attributes][:user_id]
            if user && user.update(lessor_id: @instance.id) 
              return render json: { data: 'user updated' }
            end
          end
          
          render_json_error(:not_found, :object_not_found) unless @instance
        end

        def remove_user
          if @instance.writable_by? current_user
            user = ::User.find params[:data][:attributes][:user_id]
            if user && user.update(lessor_id: nil) 
              return render json: { data: 'user updated' }
            end
          end
          
          render_json_error(:not_found, :object_not_found) unless @instance
        end
      end
    end
  end
end
