# frozen_string_literal: true

module Hermes
  # HousingOffer serializer
  class HousingOfferSerializer < Hermes::ApplicationSerializer
    attributes %i[
    is_from_form
      requalification
      dalo
      als_id
      entry_date
      external_ref
      communicate_exact_address
      eligibility_query
      typology
      type
      age
      district
      department
      pictures
      pictures_nb
      ordered_picture_ids
      main_picture
      publication_end_date
      expiry_days
      rent_amount
      rent_with_charges
      surface
      rooms
      exclusivity_end_date
      nb_of_eligible_employees
      nb_of_requests_from_employees
      availability_date
      description
      details
      effort_rate
      designation_date_end
      min_effort_treshold
      single_parent
      reserved
      already_rent
      reservation_agreement_ref
      priority_public
      mobility
      bedrooms
      bathrooms
      heating_type
      hot_water_distribution
      application_following_contact
      floor
      floors
      elevators
      has_elevator
      has_gardian
      outside
      cellars
      has_cellar
      parking_type
      pmr
      kitchen_type
      address
      postal_code
      country
      latitude
      longitude
      address_complements
      mutation
      threshold_type
      threshold_subtype
      individual_heating
      date_publication_start
      date_publication_end
      kind
      building_type
      guarantee_deposit
      rental_charges
      parking_charges
      offer_status
      offer_status_id
      offer_status_updated_at
      lessor_status
      lessor_status_id
      lessor_status_updated_at
      ceil_category_1
      ceil_category_2
      ceil_category_3
      ceil_category_4
      ceil_category_5
      ceil_category_6
      ceil_category_7
      ceil_category_8
      ceil_category_9
      ceil_category_10
      ceil_category_11
      ceil_category_12
      floor_category_1
      floor_category_2
      floor_category_3
      floor_category_4
      floor_category_5
      floor_category_6
      floor_category_7
      floor_category_8
      floor_category_9
      floor_category_10
      floor_category_11
      floor_category_12
      min_revenues
      total_charges
      min_effort_rate
      tense_area
      subsidiary_company
      rpls_id
      operation_name
      administrator
      reserving_business_name
      reserving_siren
      administration_area
      potential_request_name_town
      potential_request_name_county
      number_of_companies_30_minutes_around
      construction_type
      construction_year
      address_can_be_shared
      qpv
      total_floors
      address_extended
      test_created_at
      already_inhabited
      new_offer
      lessor_name
      lessor_siren
      lessor_orientation
      als_siren
      first_quartile
      postal_code
      insee_code
      residence_code
      residence_title
      agency_id
      agency_name
      lessor_contact_name
      lessor_email
      lessor_phone_number
      gardens
      has_garden
      green_area
      terraces_or_balconies
      dpe_ges
      dpe_conso
      condo
      furnished_lodging
      virtual_visit
      duplex
      first_quartile
      other_quartiles
      lessor_convention_reference
      reserving_agency_name
      reserving_agency_siren
      disability_accesses
      has_disability_access
      profile_epci_score
      score_with_position
      applicated_position
      eligible_position
      effort_rate
      lessor_id
      lessor
      entities_siret
      entity_ids
      threshold_id
      agency_id
      eligible_requests_nb
      applicated_nb
      reference
      release_date
      commercialized
      is_public
      is_mad
      is_mad_updated_at
      mad_als_date
      target_objective_ids
      auto_destroy_next_day
      last_update_by_lessor
      housing_channel_keys
      accept_low_occupation
      active_reservations_rex_ids
               last_xml_validation_report
               last_send_has_error
               last_error_responses
    ]
    
    # attributes [
    #   :pictures,
    #   :pictures_nb,
    #   :ordered_picture_ids,
    #   :main_picture
    # ]
    
    # lessor_id
    # lessor
    # entity_ids
    # threshold_id
    # agency_id
    # eligible_requests_nb
    # applicated_nb
    
    type 'housing_offers'
    
    def eligible_requests_nb
      object.eligible_requests_nb
    end
    
    def applicated_nb
      object.applicated_nb
    end
    
    def lessor_id
      object.lessor_id&.to_s
    end
    
    def entity_ids
      object.entity_ids&.map(&:to_s)
    end
    
    def target_objective_ids
      object.target_objective_ids&.map(&:to_s)
    end
    
    def agency_id
      object.agency_id&.to_s
    end
    
    # def agency_name
    #   object.agency&.title
    # end
    
    def threshold_id
      object.threshold_id&.to_s
    end
    
    def offer_status_id
      object.offer_status_id.to_s
    end
    
    def lessor_status_id
      object.lessor_status_id.to_s
    end
    
    def score
      # Rails.logger.debug "TEST " + (t = Time.now).to_s
      return unless instance_options[:scores]
      
      @score ||= instance_options[:scores].find_by(housing_offer: object) # { |score| score.housing_offer_id == object.id }.first
    end
    
    def handle_score_value(value)
      value.nil? ? 'uneligible' : value
    end
    
    def profile_epci_score
      handle_score_value score&.profile_epci_score
    end
    
    def score_with_position
      handle_score_value score&.score_with_position
    end
    
    def applicated_position
      handle_score_value score&.applicated_position
    end
    
    def eligible_position
      handle_score_value score&.eligible_position
    end
    
    def effort_rate
      handle_score_value score&.effort_rate
    end
    
    def lessor
      if object.lessor
        Hermes::LessorSerializer.new(
          object.lessor, scope:
          scope, root: false,
          reduced: true)
      end
    end
    
    # pictures
    def ordered_picture_ids
      object.ordered_picture_ids&.map(&:to_s) || []
    end
    
    def main_picture
      # if object.ordered_picture_ids && object.ordered_picture_ids.length > 0
      #   Hermes::PictureSerializer.new(
      #     object.pictures.find(id: object.ordered_picture_ids[0].to_s),
      #     scope: scope,
      #     root: false
      #   )
      # end
      main = object.pictures.first
      return unless main
      
      Hermes::PictureSerializer.new(main, scope: scope, root: false)
    end
    
    def pictures
      object.ordered_pictures.map do |picture|
        next unless picture
        
        Hermes::PictureSerializer.new(picture, scope: scope, root: false)
      end
    end
    
    def pictures_nb
      ordered_picture_ids.length
    end
    
    def entities
      object.entities.map do |entity|
        Hermes::EntitySerializer.new(entity, scope: scope, root: false)
      end
    end
    
    def threshold
      if object.threshold
        Hermes::LessorSerializer.new(object.threshold, scope: scope, root: false)
      end
    end
  end
end
